@@ -6,16 +6,16 @@ from __future__ import division, absolute_import, unicode_literals
 import copy
 import os
 
-from cola import core
-from cola import git
-from cola import gitcmds
-from cola import gitcfg
-from cola.git import STDOUT
-from cola.observable import Observable
-from cola.decorators import memoize
-from cola.models.selection import selection_model
-from cola.models import prefs
-from cola.compat import ustr
+from .. import core
+from .. import git
+from .. import gitcmds
+from .. import gitcfg
+from ..git import STDOUT
+from ..observable import Observable
+from ..decorators import memoize
+from ..models.selection import selection_model
+from ..models import prefs
+from ..compat import ustr
 
 
 @memoize
@@ -37,11 +37,12 @@ class MainModel(Observable):
     message_updated = 'updated'
 
     # States
-    mode_none = 'none' # Default: nothing's happened, do nothing
-    mode_worktree = 'worktree' # Comparing index to worktree
-    mode_untracked = 'untracked' # Dealing with an untracked file
-    mode_index = 'index' # Comparing index to last commit
-    mode_amend = 'amend' # Amending a commit
+    mode_none = 'none'  # Default: nothing's happened, do nothing
+    mode_worktree = 'worktree'  # Comparing index to worktree
+    mode_diffstat = 'diffstat'  # Showing a diffstat
+    mode_untracked = 'untracked'  # Dealing with an untracked file
+    mode_index = 'index'  # Comparing index to last commit
+    mode_amend = 'amend'  # Amending a commit
 
     # Modes where we can checkout files from the $head
     modes_undoable = set((mode_amend, mode_index, mode_worktree))
@@ -52,7 +53,8 @@ class MainModel(Observable):
     # Modes where we can partially unstage files
     modes_unstageable = set((mode_amend, mode_index))
 
-    unstaged = property(lambda self: self.modified + self.unmerged + self.untracked)
+    unstaged = property(
+            lambda self: self.modified + self.unmerged + self.untracked)
     """An aggregate of the modified, unmerged, and untracked file lists."""
 
     def __init__(self, cwd=None):
@@ -78,7 +80,7 @@ class MainModel(Observable):
 
         self.commitmsg = ''  # current commit message
         self._auto_commitmsg = ''  # e.g. .git/MERGE_MSG
-        self._prev_commitmsg = '' # saved here when clobbered by .git/MERGE_MSG
+        self._prev_commitmsg = ''  # saved here when clobbered by .git/MERGE_MSG
 
         self.modified = []  # modified, staged, untracked, unmerged paths
         self.staged = []
@@ -128,12 +130,17 @@ class MainModel(Observable):
         if notify:
             self.notify_observers(self.message_commit_message_changed, msg)
 
-    def save_commitmsg(self, msg):
+    def save_commitmsg(self, msg=None):
+        if msg is None:
+            msg = self.commitmsg
         path = self.git.git_path('GIT_COLA_MSG')
         try:
+            if not msg.endswith('\n'):
+                msg += '\n'
             core.write(path, msg)
         except:
             pass
+        return path
 
     def set_diff_text(self, txt):
         self.diff_text = txt
@@ -242,7 +249,11 @@ class MainModel(Observable):
             self.set_mode(self.mode_none)
 
     def _update_commitmsg(self):
-        """Check for git merge message files, or clear it when the merge completes"""
+        """Check for merge message files and update the commit message
+
+        The message is cleared when the merge completes
+
+        """
         if self.amending():
             return
         # Check if there's a message file in .git/
@@ -321,9 +332,10 @@ class MainModel(Observable):
 
         return (status, '\n'.join(outs), '\n'.join(errs))
 
-    def _sliced_add(self, input_items):
-        lambda_fn = lambda x: self.git.add('--', force=True, verbose=True, *x)
-        return self._sliced_op(input_items, lambda_fn)
+    def _sliced_add(self, items):
+        add = self.git.add
+        return self._sliced_op(
+                items, lambda x: add('--', force=True, verbose=True, *x))
 
     def stage_modified(self):
         status, out, err = self._sliced_add(self.modified)
@@ -336,8 +348,8 @@ class MainModel(Observable):
         return (status, out, err)
 
     def reset(self, *items):
-        lambda_fn = lambda x: self.git.reset('--', *x)
-        status, out, err = self._sliced_op(items, lambda_fn)
+        reset = self.git.reset
+        status, out, err = self._sliced_op(items, lambda x: reset('--', *x))
         self.update_file_status()
         return (status, out, err)
 
@@ -369,7 +381,7 @@ class MainModel(Observable):
 
         kwargs = {
             'list': True,
-            'global': not local, # global is a python keyword
+            'global': not local,  # global is a python keyword
         }
         config_lines = self.git.config(**kwargs)[STDOUT].splitlines()
         newdict = {}
@@ -379,14 +391,14 @@ class MainModel(Observable):
             except:
                 # value-less entry in .gitconfig
                 continue
-            k = k.replace('.','_') # git -> model
+            k = k.replace('.', '_')  # git -> model
             if v == 'true' or v == 'false':
                 v = bool(eval(v.title()))
             try:
                 v = int(eval(v))
             except:
                 pass
-            newdict[k]=v
+            newdict[k] = v
         return newdict
 
     def remote_url(self, name, action):
@@ -402,8 +414,9 @@ class MainModel(Observable):
 
     def push(self, remote, remote_branch='', local_branch='', **opts):
         # Swap the branches in push mode (reverse of fetch)
-        opts.update(dict(local_branch=remote_branch, remote_branch=local_branch))
-        return run_remote_action(self.git.push, remote, **opts)
+        opts.update(dict(local_branch=remote_branch,
+                         remote_branch=local_branch))
+        return run_remote_action(self.git.push, remote, push=True, **opts)
 
     def pull(self, remote, **opts):
         return run_remote_action(self.git.pull, remote, pull=True, **opts)
@@ -495,35 +508,54 @@ class MainModel(Observable):
 def remote_args(remote,
                 local_branch='',
                 remote_branch='',
-                ffwd=True,
+                ff_only=False,
+                force=False,
+                no_ff=False,
                 tags=False,
                 rebase=False,
-                pull=False):
+                pull=False,
+                push=False,
+                set_upstream=False):
     """Return arguments for git fetch/push/pull"""
 
     args = [remote]
-    what = refspec_arg(local_branch, remote_branch, ffwd, pull)
+    what = refspec_arg(local_branch, remote_branch, pull, push)
     if what:
         args.append(what)
+
     kwargs = {
         'verbose': True,
-        'tags': tags,
-        'rebase': rebase,
     }
+    if pull:
+        if rebase:
+            kwargs['rebase'] = True
+        elif ff_only:
+            kwargs['ff_only'] = True
+        elif no_ff:
+            kwargs['no_ff'] = True
+    elif force:
+        kwargs['force'] = True
+
+    if push and set_upstream:
+        kwargs['set_upstream'] = True
+    if tags:
+        kwargs['tags'] = True
+
     return (args, kwargs)
 
 
-def refspec(src, dst, ffwd):
-    spec = '%s:%s' % (src, dst)
-    if not ffwd:
-        spec = '+' + spec
+def refspec(src, dst, push=False):
+    if push and src == dst:
+        spec = src
+    else:
+        spec = '%s:%s' % (src, dst)
     return spec
 
 
-def refspec_arg(local_branch, remote_branch, ffwd, pull):
+def refspec_arg(local_branch, remote_branch, pull, push):
     """Return the refspec for a fetch or pull command"""
     if not pull and local_branch and remote_branch:
-        what = refspec(remote_branch, local_branch, ffwd)
+        what = refspec(remote_branch, local_branch, push=push)
     else:
         what = local_branch or remote_branch or None
     return what
