@@ -1,24 +1,23 @@
 """Provides dialogs for comparing branches and commits."""
 from __future__ import division, absolute_import, unicode_literals
 
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
 
-from cola import qtutils
-from cola import difftool
-from cola import gitcmds
-from cola import icons
-from cola.i18n import N_
-from cola.qtutils import connect_button
-from cola.widgets import defs
-from cola.widgets import standard
+from .. import cmds
+from .. import gitcmds
+from .. import icons
+from .. import qtutils
+from ..i18n import N_
+from ..qtutils import connect_button
+from . import defs
+from . import standard
 
 
-class FileItem(QtGui.QTreeWidgetItem):
+class FileItem(QtWidgets.QTreeWidgetItem):
 
     def __init__(self, path, icon):
-        QtGui.QTreeWidgetItem.__init__(self, [path])
+        QtWidgets.QTreeWidgetItem.__init__(self, [path])
         self.path = path
         self.setIcon(0, icon)
 
@@ -32,7 +31,6 @@ def compare_branches():
 
 class CompareBranchesDialog(standard.Dialog):
 
-
     def __init__(self, parent):
         standard.Dialog.__init__(self, parent=parent)
 
@@ -45,25 +43,25 @@ class CompareBranchesDialog(standard.Dialog):
         self.remote_branches = gitcmds.branch_list(remote=True)
         self.local_branches = gitcmds.branch_list(remote=False)
 
-        self.top_widget = QtGui.QWidget()
-        self.bottom_widget = QtGui.QWidget()
+        self.top_widget = QtWidgets.QWidget()
+        self.bottom_widget = QtWidgets.QWidget()
 
-        self.left_combo = QtGui.QComboBox()
+        self.left_combo = QtWidgets.QComboBox()
         self.left_combo.addItem(N_('Local'))
         self.left_combo.addItem(N_('Remote'))
         self.left_combo.setCurrentIndex(0)
 
-        self.right_combo = QtGui.QComboBox()
+        self.right_combo = QtWidgets.QComboBox()
         self.right_combo.addItem(N_('Local'))
         self.right_combo.addItem(N_('Remote'))
         self.right_combo.setCurrentIndex(1)
 
-        self.left_list = QtGui.QListWidget()
-        self.right_list = QtGui.QListWidget()
+        self.left_list = QtWidgets.QListWidget()
+        self.right_list = QtWidgets.QListWidget()
 
-        self.button_spacer = QtGui.QSpacerItem(1, 1,
-                                               QtGui.QSizePolicy.Expanding,
-                                               QtGui.QSizePolicy.Minimum)
+        Expanding = QtWidgets.QSizePolicy.Expanding
+        Minimum = QtWidgets.QSizePolicy.Minimum
+        self.button_spacer = QtWidgets.QSpacerItem(1, 1, Expanding, Minimum)
 
         self.button_compare = qtutils.create_button(text=N_('Compare'),
                                                     icon=icons.diff())
@@ -91,30 +89,22 @@ class CompareBranchesDialog(standard.Dialog):
         self.splitter = qtutils.splitter(Qt.Vertical,
                                          self.top_widget, self.bottom_widget)
 
-        self.main_layout = qtutils.vbox(defs.margin, defs.spacing, self.splitter)
+        self.main_layout = qtutils.vbox(defs.margin, defs.spacing,
+                                        self.splitter)
         self.setLayout(self.main_layout)
         self.resize(658, 350)
 
         connect_button(self.button_close, self.accept)
         connect_button(self.button_compare, self.compare)
 
-        self.connect(self.diff_files,
-                     SIGNAL('itemDoubleClicked(QTreeWidgetItem*,int)'),
-                     self.compare)
+        self.diff_files.itemDoubleClicked.connect(self.compare)
+        self.left_combo.currentIndexChanged.connect(
+                lambda x: self.update_combo_boxes(left=True))
+        self.right_combo.currentIndexChanged.connect(
+                lambda x: self.update_combo_boxes(left=False))
 
-        self.connect(self.left_combo,
-                     SIGNAL('currentIndexChanged(int)'),
-                     lambda x: self.update_combo_boxes(left=True))
-
-        self.connect(self.right_combo,
-                     SIGNAL('currentIndexChanged(int)'),
-                     lambda x: self.update_combo_boxes(left=False))
-
-        self.connect(self.left_list,
-                     SIGNAL('itemSelectionChanged()'), self.update_diff_files)
-
-        self.connect(self.right_list,
-                     SIGNAL('itemSelectionChanged()'), self.update_diff_files)
+        self.left_list.itemSelectionChanged.connect(self.update_diff_files)
+        self.right_list.itemSelectionChanged.connect(self.update_diff_files)
 
         self.update_combo_boxes(left=True)
         self.update_combo_boxes(left=False)
@@ -123,12 +113,12 @@ class CompareBranchesDialog(standard.Dialog):
         item = self.left_list.item(0)
         if item:
             self.left_list.setCurrentItem(item)
-            self.left_list.setItemSelected(item, True)
+            item.setSelected(True)
 
         item = self.right_list.item(0)
         if item:
             self.right_list.setCurrentItem(item)
-            self.right_list.setItemSelected(item, True)
+            item.setSelected(True)
 
     def selection(self):
         left_item = self.left_list.currentItem()
@@ -143,7 +133,6 @@ class CompareBranchesDialog(standard.Dialog):
             right_item = None
         return (left_item, right_item)
 
-
     def update_diff_files(self, *rest):
         """Updates the list of files whenever the selection changes"""
         # Left and Right refer to the comparison pair (l,r)
@@ -208,7 +197,6 @@ class CompareBranchesDialog(standard.Dialog):
             # Compare against the remote branch
             return branch
 
-
     def update_combo_boxes(self, left=False):
         """Update listwidgets from the combobox selection
 
@@ -227,7 +215,7 @@ class CompareBranchesDialog(standard.Dialog):
         # sandbox as a valid choice.  If we're looking at
         # "remote" stuff then also include the branch point.
         if which == self.LOCAL:
-            new_list = ([self.SANDBOX]+ self.local_branches)
+            new_list = ([self.SANDBOX] + self.local_branches)
         else:
             new_list = ([self.BRANCH_POINT] + self.remote_branches)
 
@@ -236,7 +224,7 @@ class CompareBranchesDialog(standard.Dialog):
         if new_list:
             item = widget.item(0)
             widget.setCurrentItem(item)
-            widget.setItemSelected(item, True)
+            item.setSelected(True)
 
     def compare(self, *args):
         """Shows the diff for a specific file
@@ -256,4 +244,4 @@ class CompareBranchesDialog(standard.Dialog):
                 right = None
         else:
             left, right = self.start, self.end
-        difftool.launch(left=left, right=right, paths=[filename])
+        cmds.difftool_launch(left=left, right=right, paths=[filename])
