@@ -27,7 +27,7 @@ class FileWidget(TreeWidget):
                 self, N_('Show History'), self.show_history, hotkeys.HISTORY)
 
         self.launch_difftool_action = qtutils.add_action(
-                self, N_('Launch Diff Tool'), self.show_diff, hotkeys.DIFF)
+                self, N_('Launch Diff Tool'), self.show_diff)
 
         self.launch_editor_action = qtutils.add_action(
                 self, N_('Launch Diff Tool'), self.edit_paths, hotkeys.EDIT)
@@ -43,8 +43,8 @@ class FileWidget(TreeWidget):
         if not commits:
             return
         commit = commits[0]
-        sha1 = commit.sha1
-        status, out, err = git.show(sha1, z=True, numstat=True,
+        oid = commit.oid
+        status, out, err = git.show(oid, z=True, numstat=True,
                                     oneline=True, no_renames=True)
         if status == 0:
             paths = [f for f in out.rstrip('\0').split('\0') if f]
