@@ -1,22 +1,21 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import fnmatch
 
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtGui
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
 
-from cola import gitcmds
-from cola import icons
-from cola import qtutils
-from cola import utils
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import main
-from cola.qtutils import connect_button
-from cola.widgets import defs
-from cola.widgets import standard
-from cola.widgets.standard import ProgressDialog
+from ..i18n import N_
+from ..interaction import Interaction
+from ..models import main
+from ..qtutils import connect_button
+from .. import gitcmds
+from .. import icons
+from .. import qtutils
+from .. import utils
+from .standard import ProgressDialog
+from . import defs
+from . import standard
 
 
 FETCH = 'FETCH'
@@ -72,6 +71,11 @@ def combine(result, existing):
             return result
 
 
+def disable(value, *checkboxes):
+    if value:
+        for checkbox in checkboxes:
+            checkbox.setChecked(False)
+
 class ActionTask(qtutils.Task):
 
     def __init__(self, parent, model_action, remote, kwargs):
@@ -96,7 +100,6 @@ class RemoteActionDialog(standard.Dialog):
         self.filtered_remote_branches = []
         self.selected_remotes = []
 
-        self.setAttribute(Qt.WA_MacMetalStyle)
         self.setWindowTitle(title)
         if parent is not None:
             self.setWindowModality(Qt.WindowModal)
@@ -104,60 +107,100 @@ class RemoteActionDialog(standard.Dialog):
         self.runtask = qtutils.RunTask(parent=self)
         self.progress = ProgressDialog(title, N_('Updating'), self)
 
-        self.local_label = QtGui.QLabel()
+        self.local_label = QtWidgets.QLabel()
         self.local_label.setText(N_('Local Branch'))
 
-        self.local_branch = QtGui.QLineEdit()
-        self.local_branches = QtGui.QListWidget()
+        self.local_branch = QtWidgets.QLineEdit()
+        self.local_branches = QtWidgets.QListWidget()
         self.local_branches.addItems(self.model.local_branches)
 
-        self.remote_label = QtGui.QLabel()
+        self.remote_label = QtWidgets.QLabel()
         self.remote_label.setText(N_('Remote'))
 
-        self.remote_name = QtGui.QLineEdit()
-        self.remotes = QtGui.QListWidget()
+        self.remote_name = QtWidgets.QLineEdit()
+        self.remotes = QtWidgets.QListWidget()
         if action == PUSH:
-            self.remotes.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+            mode = QtWidgets.QAbstractItemView.ExtendedSelection
+            self.remotes.setSelectionMode(mode)
         self.remotes.addItems(self.model.remotes)
 
-        self.remote_branch_label = QtGui.QLabel()
+        self.remote_branch_label = QtWidgets.QLabel()
         self.remote_branch_label.setText(N_('Remote Branch'))
 
-        self.remote_branch = QtGui.QLineEdit()
-        self.remote_branches = QtGui.QListWidget()
+        self.remote_branch = QtWidgets.QLineEdit()
+        self.remote_branches = QtWidgets.QListWidget()
         self.remote_branches.addItems(self.model.remote_branches)
 
-        text = N_('Fast Forward Only ')
-        self.ffwd_only_checkbox = qtutils.checkbox(text=text, checked=True)
-        self.tags_checkbox = qtutils.checkbox(text=N_('Include tags '))
-        self.rebase_checkbox = qtutils.checkbox(text=N_('Rebase '))
+        text = N_('Prompt on creation')
+        tooltip = N_('Prompt when pushing creates new remote branches')
+        self.prompt_checkbox = qtutils.checkbox(checked=True, text=text,
+                                                tooltip=tooltip)
 
-        if icon is None:
-            icon = icons.ok()
-        self.action_button = qtutils.create_button(text=title, icon=icon)
+        text = N_('Fast-forward only')
+        tooltip = N_('Refuse to merge unless the current HEAD is already up-'
+                     'to-date or the merge can be resolved as a fast-forward')
+        self.ff_only_checkbox = qtutils.checkbox(checked=True,
+                                                 text=text, tooltip=tooltip)
+
+        text = N_('No fast-forward')
+        tooltip = N_('Create a merge commit even when the merge resolves as a '
+                     'fast-forward')
+        self.no_ff_checkbox = qtutils.checkbox(checked=False,
+                                               text=text, tooltip=tooltip)
+        text = N_('Force')
+        tooltip = N_('Allow non-fast-forward updates.  Using "force" can '
+                     'cause the remote repository to lose commits; '
+                     'use it with care')
+        self.force_checkbox = qtutils.checkbox(checked=False, text=text,
+                                               tooltip=tooltip)
+
+        self.tags_checkbox = qtutils.checkbox(text=N_('Include tags '))
+
+        tooltip = N_('Rebase the current branch instead of merging')
+        self.rebase_checkbox = qtutils.checkbox(text=N_('Rebase'),
+                                                tooltip=tooltip)
+
+        text = N_('Set upstream')
+        tooltip = N_('Configure the remote branch as the the new upstream')
+        self.upstream_checkbox = qtutils.checkbox(text=text, tooltip=tooltip)
+
+        self.action_button = qtutils.ok_button(title, icon=icon)
         self.close_button = qtutils.close_button()
 
         self.buttons = utils.Group(self.action_button, self.close_button)
 
-        self.local_branch_layout = qtutils.hbox(defs.small_margin, defs.spacing,
-                                                self.local_label,
-                                                self.local_branch)
+        self.local_branch_layout = qtutils.hbox(
+                defs.small_margin,
+                defs.spacing,
+                self.local_label,
+                self.local_branch)
 
-        self.remote_branch_layout = qtutils.hbox(defs.small_margin, defs.spacing,
-                                                 self.remote_label,
-                                                 self.remote_name)
+        self.remote_branch_layout = qtutils.hbox(
+                defs.small_margin,
+                defs.spacing,
+                self.remote_label,
+                self.remote_name)
 
-        self.remote_branches_layout = qtutils.hbox(defs.small_margin, defs.spacing,
-                                                   self.remote_branch_label,
-                                                   self.remote_branch)
+        self.remote_branches_layout = qtutils.hbox(
+                defs.small_margin,
+                defs.spacing,
+                self.remote_branch_label,
+                self.remote_branch)
+
+        self.options_layout = qtutils.hbox(
+                defs.no_margin,
+                defs.button_spacing,
+                self.ff_only_checkbox,
+                self.no_ff_checkbox,
+                self.tags_checkbox,
+                self.rebase_checkbox,
+                self.upstream_checkbox,
+                self.prompt_checkbox,
+                self.force_checkbox,
+                qtutils.STRETCH,
+                self.close_button,
+                self.action_button)
 
-        self.options_layout = qtutils.hbox(defs.no_margin, defs.button_spacing,
-                                           qtutils.STRETCH,
-                                           self.ffwd_only_checkbox,
-                                           self.tags_checkbox,
-                                           self.rebase_checkbox,
-                                           self.action_button,
-                                           self.close_button)
         if action == PUSH:
             widgets = (
                     self.remote_branch_layout, self.remotes,
@@ -165,7 +208,7 @@ class RemoteActionDialog(standard.Dialog):
                     self.remote_branches_layout, self.remote_branches,
                     self.options_layout,
             )
-        else: # fetch and pull
+        else:  # fetch and pull
             widgets = (
                     self.remote_branch_layout, self.remotes,
                     self.remote_branches_layout, self.remote_branches,
@@ -191,14 +234,23 @@ class RemoteActionDialog(standard.Dialog):
         self.set_field_defaults()
 
         # Setup signals and slots
-        self.connect(self.remotes, SIGNAL('itemSelectionChanged()'),
-                     self.update_remotes)
+        self.remotes.itemSelectionChanged.connect(self.update_remotes)
 
-        self.connect(self.local_branches, SIGNAL('itemSelectionChanged()'),
-                     self.update_local_branches)
+        local = self.local_branches
+        local.itemSelectionChanged.connect(self.update_local_branches)
 
-        self.connect(self.remote_branches, SIGNAL('itemSelectionChanged()'),
-                     self.update_remote_branches)
+        remote = self.remote_branches
+        remote.itemSelectionChanged.connect(self.update_remote_branches)
+
+        self.no_ff_checkbox.toggled.connect(
+                lambda x: disable(x, self.ff_only_checkbox,
+                                  self.rebase_checkbox))
+        self.ff_only_checkbox.toggled.connect(
+                lambda x: disable(x, self.no_ff_checkbox,
+                                  self.rebase_checkbox))
+        self.rebase_checkbox.toggled.connect(
+                lambda x: disable(x, self.no_ff_checkbox,
+                                  self.ff_only_checkbox))
 
         connect_button(self.action_button, self.action_callback)
         connect_button(self.close_button, self.close)
@@ -206,19 +258,29 @@ class RemoteActionDialog(standard.Dialog):
         qtutils.add_action(self, N_('Close'), self.close,
                            QtGui.QKeySequence.Close, 'Esc')
 
+        if action != PUSH:
+            # Push-only options
+            self.upstream_checkbox.hide()
+            self.prompt_checkbox.hide()
+
         if action == PULL:
+            # Fetch and Push-only options
+            self.force_checkbox.hide()
             self.tags_checkbox.hide()
-            self.ffwd_only_checkbox.hide()
             self.local_label.hide()
             self.local_branch.hide()
             self.local_branches.hide()
             self.remote_branch.setFocus()
         else:
+            # Pull-only options
             self.rebase_checkbox.hide()
+            self.no_ff_checkbox.hide()
+            self.ff_only_checkbox.hide()
 
-        if not self.restore_state():
-            self.resize(666, 420)
-
+        desktop = qtutils.desktop()
+        width = desktop.width()/2
+        height = desktop.height() - desktop.height()/4
+        self.init_state(None, self.resize, width, height)
         self.remote_name.setFocus()
 
     def set_rebase(self, value):
@@ -271,7 +333,7 @@ class RemoteActionDialog(standard.Dialog):
         """Selects a remote by index"""
         item = self.remotes.item(idx)
         if item:
-            self.remotes.setItemSelected(item, True)
+            item.setSelected(True)
             self.remotes.setCurrentItem(item)
             self.set_remote_name(item.text())
             return True
@@ -283,7 +345,7 @@ class RemoteActionDialog(standard.Dialog):
         item = self.local_branches.item(idx)
         if not item:
             return False
-        self.local_branches.setItemSelected(item, True)
+        item.setSelected(True)
         self.local_branches.setCurrentItem(item)
         self.local_branch.setText(item.text())
         return True
@@ -296,7 +358,7 @@ class RemoteActionDialog(standard.Dialog):
             display = ('%s\t(%s)'
                        % (remote_name, N_('URL: %s') % url))
             displayed.append(display)
-        qtutils.set_items(widget,displayed)
+        qtutils.set_items(widget, displayed)
 
     def update_remotes(self, *rest):
         """Update the remote name when a remote from the list is selected"""
@@ -328,7 +390,7 @@ class RemoteActionDialog(standard.Dialog):
             self.set_remote_branches(all_branches)
         self.set_remote_branch('')
 
-    def update_local_branches(self,*rest):
+    def update_local_branches(self, *rest):
         """Update the local/remote branch names when a branch is selected"""
         branches = self.model.local_branches
         widget = self.local_branches
@@ -338,7 +400,7 @@ class RemoteActionDialog(standard.Dialog):
         self.set_local_branch(selection)
         self.set_remote_branch(selection)
 
-    def update_remote_branches(self,*rest):
+    def update_remote_branches(self, *rest):
         """Update the remote branch name when a branch is selected"""
         widget = self.remote_branches
         branches = self.filtered_remote_branches
@@ -351,21 +413,27 @@ class RemoteActionDialog(standard.Dialog):
         self.set_remote_branch(branch)
 
     def common_args(self):
-        """Returns git arguments common to fetch/push/pulll"""
+        """Returns git arguments common to fetch/push/pull"""
         remote_name = self.remote_name.text()
         local_branch = self.local_branch.text()
         remote_branch = self.remote_branch.text()
 
-        ffwd_only = self.ffwd_only_checkbox.isChecked()
+        ff_only = self.ff_only_checkbox.isChecked()
+        force = self.force_checkbox.isChecked()
+        no_ff = self.no_ff_checkbox.isChecked()
         rebase = self.rebase_checkbox.isChecked()
+        set_upstream = self.upstream_checkbox.isChecked()
         tags = self.tags_checkbox.isChecked()
 
         return (remote_name,
                 {
+                    'ff_only': ff_only,
+                    'force': force,
                     'local_branch': local_branch,
-                    'remote_branch': remote_branch,
-                    'ffwd': ffwd_only,
+                    'no_ff': no_ff,
                     'rebase': rebase,
+                    'remote_branch': remote_branch,
+                    'set_upstream': set_upstream,
                     'tags': tags,
                 })
 
@@ -385,7 +453,7 @@ class RemoteActionDialog(standard.Dialog):
             model_action = self.model.fetch
         elif action == PUSH:
             model_action = self.push_to_all
-        else: # if action == PULL:
+        else:  # if action == PULL:
             model_action = self.model.pull
 
         remote_name = self.remote_name.text()
@@ -404,18 +472,20 @@ class RemoteActionDialog(standard.Dialog):
         if action == PUSH and not remote_branch:
             branch = local_branch
             candidate = '%s/%s' % (remote, branch)
-            if candidate not in self.model.remote_branches:
+            prompt = self.prompt_checkbox.isChecked()
+
+            if prompt and candidate not in self.model.remote_branches:
                 title = N_('Push')
                 args = dict(branch=branch, remote=remote)
                 msg = N_('Branch "%(branch)s" does not exist in "%(remote)s".\n'
                          'A new remote branch will be published.') % args
-                info_txt= N_('Create a new remote branch?')
+                info_txt = N_('Create a new remote branch?')
                 ok_text = N_('Create Remote Branch')
                 if not qtutils.confirm(title, msg, info_txt, ok_text,
                                        icon=icons.cola()):
                     return
 
-        if not self.ffwd_only_checkbox.isChecked():
+        if self.force_checkbox.isChecked():
             if action == FETCH:
                 title = N_('Force Fetch?')
                 msg = N_('Non-fast-forward fetch overwrites local history!')
@@ -427,10 +497,8 @@ class RemoteActionDialog(standard.Dialog):
                          'history!\n(Did you pull first?)')
                 info_txt = N_('Force push to %s?') % remote
                 ok_text = N_('Force Push')
-            else: # pull: shouldn't happen since the controls are hidden
-                msg = "You probably don't want to do this.\n\tContinue?"
+            else:  # pull: shouldn't happen since the controls are hidden
                 return
-
             if not qtutils.confirm(title, msg, info_txt, ok_text,
                                    default=False, icon=icons.discard()):
                 return
@@ -451,7 +519,7 @@ class RemoteActionDialog(standard.Dialog):
 
         already_up_to_date = N_('Already up-to-date.')
 
-        if not out: # git fetch --tags --verbose doesn't print anything...
+        if not out:  # git fetch --tags --verbose doesn't print anything...
             out = already_up_to_date
 
         command = 'git %s' % self.action.lower()
@@ -487,6 +555,17 @@ class Fetch(RemoteActionDialog):
         RemoteActionDialog.__init__(self, model, FETCH, N_('Fetch'),
                                     parent=parent, icon=icons.repo())
 
+    def export_state(self):
+        state = RemoteActionDialog.export_state(self)
+        state['tags'] = self.tags_checkbox.isChecked()
+        return state
+
+    def apply_state(self, state):
+        result = RemoteActionDialog.apply_state(self, state)
+        tags = bool(state.get('tags', False))
+        self.tags_checkbox.setChecked(tags)
+        return result
+
 
 class Push(RemoteActionDialog):
 
@@ -494,6 +573,23 @@ class Push(RemoteActionDialog):
         RemoteActionDialog.__init__(self, model, PUSH, N_('Push'),
                                     parent=parent, icon=icons.push())
 
+    def export_state(self):
+        state = RemoteActionDialog.export_state(self)
+        state['tags'] = self.tags_checkbox.isChecked()
+        state['prompt'] = self.prompt_checkbox.isChecked()
+        return state
+
+    def apply_state(self, state):
+        result = RemoteActionDialog.apply_state(self, state)
+
+        tags = bool(state.get('tags', False))
+        self.tags_checkbox.setChecked(tags)
+
+        prompt = bool(state.get('prompt', True))
+        self.prompt_checkbox.setChecked(prompt)
+
+        return result
+
 
 class Pull(RemoteActionDialog):
 
@@ -503,17 +599,29 @@ class Pull(RemoteActionDialog):
 
     def apply_state(self, state):
         result = RemoteActionDialog.apply_state(self, state)
-        try:
-            rebase = state['rebase']
-        except KeyError:
-            result = False
-        else:
-            self.rebase_checkbox.setChecked(rebase)
+        # Rebase has the highest priority
+        rebase = bool(state.get('rebase', False))
+        ff_only = not rebase and bool(state.get('ff_only', False))
+        no_ff = not rebase and not ff_only and bool(state.get('no_ff', False))
+
+        self.rebase_checkbox.setChecked(rebase)
+        self.no_ff_checkbox.setChecked(no_ff)
+
+        # Allow users coming from older versions that have rebase=False to
+        # pickup the new ff_only=True default by only setting ff_only False
+        # when it either exists in the config or when rebase=True.
+        if 'ff_only' in state or rebase:
+            self.ff_only_checkbox.setChecked(ff_only)
+
         return result
 
     def export_state(self):
         state = RemoteActionDialog.export_state(self)
+
+        state['ff_only'] = self.ff_only_checkbox.isChecked()
+        state['no_ff'] = self.no_ff_checkbox.isChecked()
         state['rebase'] = self.rebase_checkbox.isChecked()
+
         return state
 
     def done(self, exit_code):
