@@ -39,7 +39,6 @@ class SearchWidget(standard.Dialog):
 
     def __init__(self, parent):
         standard.Dialog.__init__(self, parent)
-        self.setAttribute(Qt.WA_MacMetalStyle)
         self.setWindowTitle(N_('Search'))
 
         self.mode_combo = QtWidgets.QComboBox()
@@ -303,7 +302,7 @@ class Search(SearchWidget):
             self.commit_text.setText('')
 
     def browse_callback(self):
-        paths = qtutils.open_files(N_('Choose Path(s)'))
+        paths = qtutils.open_files(N_('Choose Paths'))
         if not paths:
             return
         filepaths = []
