@@ -9,17 +9,21 @@ from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
 from qtpy.QtWidgets import QDockWidget
 
+from ..settings import Settings
 from .. import core
 from .. import gitcfg
 from .. import qtcompat
 from .. import qtutils
-from ..settings import Settings
+from .. import utils
 from . import defs
 
 
 class WidgetMixin(object):
     """Mix-in for common utilities and serialization of widget state"""
 
+    def __init__(self):
+        self._unmaximized_rect = None
+
     def center(self):
         parent = self.parent()
         if parent is None:
@@ -36,7 +40,17 @@ class WidgetMixin(object):
         desktop = QtWidgets.QApplication.instance().desktop()
         width = desktop.width()
         height = desktop.height()
-        self.resize(width, height)
+        if utils.is_darwin():
+            self.resize(width, height)
+        else:
+            shown = self.isVisible()
+            # earlier show() fools Windows focus stealing prevention. the main
+            # window is blocked for the duration of "git rebase" and we don't
+            # want to present a blocked window with git-xbase hidden somewhere.
+            self.show()
+            self.setWindowState(Qt.WindowMaximized)
+            if not shown:
+                self.hide()
 
     def name(self):
         """Returns the name of the view class"""
@@ -49,6 +63,35 @@ class WidgetMixin(object):
         if gitcfg.current().get('cola.savewindowsettings', True):
             settings.save_gui_state(self)
 
+    def resizeEvent(self, event):
+        super(WidgetMixin, self).resizeEvent(event)
+        # Use a timer to so that the window size and state is up to date.
+        # If we ask for the window state here it will never realize that
+        # we have been maximized because the window state change is processed
+        # after the resize event.  Using a timer event causes it to happen
+        # after all the events have been processsed.
+        # Timer with a delay of zero will trigger immediately after control
+        # returns to the main loop.
+        QtCore.QTimer.singleShot(
+                0, lambda: self._store_unmaximized_dimensions())
+
+    def moveEvent(self, event):
+        super(WidgetMixin, self).moveEvent(event)
+        # as per the QObject::resizeEvent() override
+        QtCore.QTimer.singleShot(
+                0, lambda: self._store_unmaximized_dimensions())
+
+    def _store_unmaximized_dimensions(self):
+        state = self.windowState()
+        maximized = bool(state & Qt.WindowMaximized)
+        if not maximized:
+            size = self.size()
+            width, height = size.width(), size.height()
+            x, y = self.x(), self.y()
+            # XXX can width and height ever not be over zero?
+            if width > 0 and height > 0:
+                self._unmaximized_rect = (x, y, width, height)
+
     def restore_state(self, settings=None):
         if settings is None:
             settings = Settings()
@@ -60,16 +103,26 @@ class WidgetMixin(object):
         """Imports data for view save/restore"""
         result = True
         try:
-            self.resize(state['width'], state['height'])
-        except:
-            result = False
-        try:
-            self.move(state['x'], state['y'])
+            width, height = int(state['width']), int(state['height'])
+            self.resize(width, height)
+
+            x, y = int(state['x']), int(state['y'])
+            self.move(x, y)
+
+            # calling resize/move won't invoke QWidget::{resize,move}Event
+            # so store the unmaximized size if we properly restored.
+            self._unmaximized_rect = (x, y, width, height)
         except:
             result = False
         try:
             if state['maximized']:
-                self.showMaximized()
+                try:
+                    if utils.is_win32() or utils.is_darwin():
+                        self.resize_to_desktop()
+                    else:
+                        self.showMaximized()
+                except:
+                    pass
         except:
             result = False
         self._apply_state_applied = result
@@ -79,14 +132,27 @@ class WidgetMixin(object):
         """Exports data for view save/restore"""
         state = self.windowState()
         maximized = bool(state & Qt.WindowMaximized)
-        return {
-            'x': self.x(),
-            'y': self.y(),
-            'width': self.width(),
-            'height': self.height(),
+
+        ret = {
             'maximized': maximized,
         }
 
+        # when maximized we don't want to overwrite saved x/y/width/height with
+        # desktop dimensions.
+        if maximized:
+            rect = self._unmaximized_rect
+            try:
+                ret['x'], ret['y'], ret['width'], ret['height'] = rect
+            except:
+                pass
+        else:
+            ret['width'] = self.width()
+            ret['height'] = self.height()
+            ret['x'] = self.x()
+            ret['y'] = self.y()
+
+        return ret
+
     def save_settings(self):
         settings = Settings()
         settings.load()
@@ -130,7 +196,7 @@ class MainWindowMixin(WidgetMixin):
         else:
             from_base64 = QtCore.QByteArray.fromBase64
             result = self.restoreState(
-                    from_base64(str(windowstate)),
+                    from_base64(core.encode(windowstate)),
                     self.widget_version) and result
         self.lock_layout = state.get('lock_layout', self.lock_layout)
         self.update_dockwidget_lock_state()
@@ -388,15 +454,31 @@ class MainWindow(MainWindowMixin, QtWidgets.QMainWindow):
         QtWidgets.QMainWindow.__init__(self, parent)
         MainWindowMixin.__init__(self)
 
+        palette = self.palette()
+        window = palette.color(QtGui.QPalette.Window)
+        highlight = palette.color(QtGui.QPalette.Highlight)
+
+        window_rgb = ('rgb(%d, %d, %d)'
+                 % (window.red(),
+                    window.blue(),
+                    window.green()))
+        highlight_rgb = ('rgb(%d, %d, %d)'
+                 % (highlight.red(),
+                    highlight.blue(),
+                    highlight.green()))
+
         self.setStyleSheet("""
             QMainWindow::separator {
+                background: %(window_rgb)s;
                 width: %(separator)spx;
                 height: %(separator)spx;
             }
             QMainWindow::separator:hover {
-                background: white;
+                background: %(highlight_rgb)s;
             }
-            """ % dict(separator=defs.separator))
+            """ % dict(separator=defs.separator,
+                       window_rgb=window_rgb,
+                       highlight_rgb=highlight_rgb))
 
 
 class TreeView(QtWidgets.QTreeView):
