@@ -1,22 +1,22 @@
 """Provides the StashView dialog."""
 from __future__ import division, absolute_import, unicode_literals
 
-from PyQt4 import QtCore
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
-from PyQt4.QtCore import SIGNAL
+from qtpy import QtCore
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
 
-from cola import cmds
-from cola import qtutils
-from cola import utils
-from cola.i18n import N_
-from cola.models.stash import StashModel
-from cola.models.stash import ApplyStash
-from cola.models.stash import SaveStash
-from cola.models.stash import DropStash
-from cola.widgets import defs
-from cola.widgets.diff import DiffTextEdit
-from cola.widgets.standard import Dialog
+from .. import cmds
+from .. import icons
+from .. import qtutils
+from .. import utils
+from ..i18n import N_
+from ..models.stash import StashModel
+from ..models.stash import ApplyStash
+from ..models.stash import SaveStash
+from ..models.stash import DropStash
+from . import defs
+from .diff import DiffTextEdit
+from .standard import Dialog
 
 
 def stash():
@@ -38,35 +38,33 @@ class StashView(Dialog):
         self.names = []
 
         self.setWindowTitle(N_('Stash'))
-        self.setAttribute(QtCore.Qt.WA_MacMetalStyle)
         if parent is not None:
             self.setWindowModality(QtCore.Qt.WindowModal)
             self.resize(parent.width(), 420)
         else:
             self.resize(700, 420)
 
-        self.stash_list = QtGui.QListWidget(self)
+        self.stash_list = QtWidgets.QListWidget(self)
         self.stash_text = DiffTextEdit(self)
 
-        self.button_apply =\
-            self.toolbutton(N_('Apply'),
-                            N_('Apply the selected stash'),
-                            qtutils.apply_icon())
-        self.button_save =\
-            self.toolbutton(N_('Save'),
-                            N_('Save modified state to new stash'),
-                            qtutils.save_icon())
-        self.button_drop = \
-            self.toolbutton(N_('Drop'),
-                            N_('Drop the selected stash'),
-                            qtutils.discard_icon())
-        self.button_close = \
-            self.pushbutton(N_('Close'),
-                            N_('Close'), qtutils.close_icon())
+        self.button_apply = qtutils.create_toolbutton(
+            text=N_('Apply'),
+            tooltip=N_('Apply the selected stash'),
+            icon=icons.ok())
 
-        self.keep_index = QtGui.QCheckBox(self)
-        self.keep_index.setText(N_('Keep Index'))
-        self.keep_index.setChecked(True)
+        self.button_save = qtutils.create_toolbutton(
+            text=N_('Save'),
+            tooltip=N_('Save modified state to new stash'),
+            icon=icons.save())
+
+        self.button_drop = qtutils.create_toolbutton(
+            text=N_('Drop'),
+            tooltip=N_('Drop the selected stash'),
+            icon=icons.discard())
+
+        self.button_close = qtutils.close_button()
+
+        self.keep_index = qtutils.checkbox(text=N_('Keep Index'), checked=True)
 
         # Arrange layouts
         self.splitter = qtutils.splitter(Qt.Horizontal,
@@ -91,8 +89,7 @@ class StashView(Dialog):
         self.setTabOrder(self.button_drop, self.keep_index)
         self.setTabOrder(self.keep_index, self.button_close)
 
-        self.connect(self.stash_list, SIGNAL('itemSelectionChanged()'),
-                     self.item_selected)
+        self.stash_list.itemSelectionChanged.connect(self.item_selected)
 
         qtutils.connect_button(self.button_apply, self.stash_apply)
         qtutils.connect_button(self.button_save, self.stash_save)
@@ -103,16 +100,6 @@ class StashView(Dialog):
         self.accept()
         cmds.do(cmds.Rescan)
 
-    def toolbutton(self, text, tooltip, icon):
-        return qtutils.create_toolbutton(text=text, tooltip=tooltip, icon=icon)
-
-    def pushbutton(self, text, tooltip, icon):
-        btn = QtGui.QPushButton(self)
-        btn.setText(text)
-        btn.setToolTip(tooltip)
-        btn.setIcon(icon)
-        return btn
-
     def selected_stash(self):
         """Returns the stash name of the currently selected stash
         """
@@ -177,8 +164,9 @@ class StashView(Dialog):
         # Sanitize the stash name
         stash_name = utils.sanitize(stash_name)
         if stash_name in self.names:
-            qtutils.critical(N_('Error: Stash exists'),
-                             N_('A stash named "%s" already exists') % stash_name)
+            qtutils.critical(
+                    N_('Error: Stash exists'),
+                    N_('A stash named "%s" already exists') % stash_name)
             return
 
         keep_index = self.keep_index.isChecked()
@@ -193,12 +181,12 @@ class StashView(Dialog):
         name = self.selected_name()
         if not selection:
             return
-        if not qtutils.confirm(N_('Drop Stash?'),
-                               N_('Recovering a dropped stash is not possible.'),
-                               N_('Drop the "%s" stash?') % name,
-                               N_('Drop Stash'),
-                               default=True,
-                               icon=qtutils.discard_icon()):
+        if not qtutils.confirm(
+                N_('Drop Stash?'),
+                N_('Recovering a dropped stash is not possible.'),
+                N_('Drop the "%s" stash?') % name,
+                N_('Drop Stash'),
+                default=True, icon=icons.discard()):
             return
         cmds.do(DropStash, selection)
         self.update_from_model()
