@@ -1,21 +1,22 @@
 from __future__ import division, absolute_import, unicode_literals
 import itertools
 
-from qtpy import QtCore
-from qtpy import QtWidgets
 from qtpy.QtCore import Qt
 from qtpy.QtCore import Signal
+from qtpy import QtCore
+from qtpy import QtWidgets
 
+from ..i18n import N_
+from ..models import main
+from ..models import prefs
+from ..models import selection
 from .. import cmds
 from .. import core
 from .. import hotkeys
 from .. import icons
 from .. import qtutils
 from .. import utils
-from ..i18n import N_
-from ..models import main
-from ..models import prefs
-from ..models import selection
+from . import common
 from . import completion
 from . import defs
 
@@ -148,15 +149,14 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         self.launch_editor_action.setIcon(icons.edit())
 
         if not utils.is_win32():
-            self.open_using_default_app = qtutils.add_action(
-                self, cmds.OpenDefaultApp.name(),
-                self._open_using_default_app, hotkeys.PRIMARY_ACTION)
-            self.open_using_default_app.setIcon(icons.default_app())
+            self.default_app_action = common.default_app_action(
+                    self, self.selected_group)
 
-            self.open_parent_dir_action = qtutils.add_action(
-                self, cmds.OpenParentDir.name(),
-                self._open_parent_dir, hotkeys.SECONDARY_ACTION)
-            self.open_parent_dir_action.setIcon(icons.folder())
+            self.parent_dir_action = common.parent_dir_action(
+                    self, self.selected_group)
+
+            self.terminal_action = common.terminal_action(
+                    self, self.selected_group)
 
         self.up_action = qtutils.add_action(
             self, N_('Move Up'), self.move_up,
@@ -183,11 +183,11 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
 
         # MoveToTrash and Delete use the same shortcut.
         # We will only bind one of them, depending on whether or not the
-        # MoveToTrash command is avaialble.  When available, the hotkey
+        # MoveToTrash command is available.  When available, the hotkey
         # is bound to MoveToTrash, otherwise it is bound to Delete.
         if cmds.MoveToTrash.AVAILABLE:
             self.move_to_trash_action = qtutils.add_action(
-                self, N_('Move file(s) to trash'),
+                self, N_('Move files to trash'),
                 self._trash_untracked_files, hotkeys.TRASH)
             self.move_to_trash_action.setIcon(icons.discard())
             delete_shortcut = hotkeys.DELETE_FILE
@@ -196,7 +196,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             delete_shortcut = hotkeys.DELETE_FILE_SECONDARY
 
         self.delete_untracked_files_action = qtutils.add_action(
-            self, N_('Delete File(s)...'),
+            self, N_('Delete Files...'),
             self._delete_untracked_files, delete_shortcut)
         self.delete_untracked_files_action.setIcon(icons.discard())
 
@@ -271,8 +271,11 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
             if category == self.idx_header:
                 item = self.invisibleRootItem().child(idx)
                 if item is not None:
+                    self.blockSignals(True)
                     self.setCurrentItem(item)
                     item.setSelected(True)
+                    self.blockSignals(False)
+                    self.show_selection()
                 return
             # Reselect the current item
             selection_info = saved_selection[category]
@@ -483,6 +486,7 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         """Set up the status menu for the repo status tree."""
         s = self.selection()
         menu = qtutils.create_menu('Status', self)
+        menu.addAction(self.launch_editor_action)
 
         selected_indexes = self.selected_indexes()
         if selected_indexes:
@@ -492,12 +496,20 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                 return self._create_header_context_menu(menu, idx)
 
         if s.staged:
-            return self._create_staged_context_menu(menu, s)
+            self._create_staged_context_menu(menu, s)
 
         elif s.unmerged:
-            return self._create_unmerged_context_menu(menu, s)
+            self._create_unmerged_context_menu(menu, s)
         else:
-            return self._create_unstaged_context_menu(menu, s)
+            self._create_unstaged_context_menu(menu, s)
+
+        if not utils.is_win32():
+            menu.addSeparator()
+            menu.addAction(self.default_app_action)
+            menu.addAction(self.parent_dir_action)
+            menu.addAction(self.terminal_action)
+
+        return menu
 
     def _create_header_context_menu(self, menu, idx):
         if idx == self.idx_staged:
@@ -535,23 +547,9 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                         for i in self.staged())
 
         if all_exist:
-            menu.addAction(self.launch_editor_action)
             menu.addAction(self.launch_difftool_action)
 
-        if all_exist and not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.staged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
-
         if self.m.undoable():
-            menu.addSeparator()
             menu.addAction(self.revert_unstaged_edits_action)
 
         menu.addSeparator()
@@ -566,9 +564,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                        cmds.run(cmds.OpenRepo,
                                 core.abspath(s.staged[0])))
 
-        menu.addAction(self.launch_editor_action)
-        menu.addSeparator()
-
         action = menu.addAction(icons.remove(), N_('Unstage Selected'),
                                 cmds.run(cmds.Unstage, self.staged()))
         action.setShortcut(hotkeys.STAGE_SELECTION)
@@ -585,20 +580,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         action = menu.addAction(icons.add(), N_('Stage Selected'),
                                 cmds.run(cmds.Stage, self.unstaged()))
         action.setShortcut(hotkeys.STAGE_SELECTION)
-        menu.addSeparator()
-        menu.addAction(self.launch_editor_action)
-
-        if not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.unmerged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
 
         menu.addSeparator()
         menu.addAction(self.copy_path_action)
@@ -622,9 +603,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         all_exist = all(i not in self.m.unstaged_deleted and core.exists(i)
                         for i in self.staged())
 
-        if all_exist and self.unstaged():
-            menu.addAction(self.launch_editor_action)
-
         if all_exist and s.modified and self.m.stageable():
             menu.addAction(self.launch_difftool_action)
 
@@ -633,18 +611,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
                 menu.addSeparator()
                 menu.addAction(self.revert_unstaged_edits_action)
 
-        if all_exist and self.unstaged() and not utils.is_win32():
-            menu.addSeparator()
-            open_default = cmds.run(cmds.OpenDefaultApp, self.unstaged())
-            action = menu.addAction(icons.default_app(),
-                                    cmds.OpenDefaultApp.name(), open_default)
-            action.setShortcut(hotkeys.PRIMARY_ACTION)
-
-            action = menu.addAction(icons.folder(),
-                                    cmds.OpenParentDir.name(),
-                                    self._open_parent_dir)
-            action.setShortcut(hotkeys.SECONDARY_ACTION)
-
         if all_exist and s.untracked:
             menu.addSeparator()
             if self.move_to_trash_action is not None:
@@ -667,8 +633,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         menu.addAction(icons.cola(), N_('Launch git-cola'),
                        cmds.run(cmds.OpenRepo, core.abspath(s.modified[0])))
 
-        menu.addAction(self.launch_editor_action)
-
         if self.m.stageable():
             menu.addSeparator()
             action = menu.addAction(icons.add(), N_('Stage Selected'),
@@ -822,12 +786,6 @@ class StatusTreeWidget(QtWidgets.QTreeWidget):
         """Called when an item is double-clicked in the repo status tree."""
         cmds.do(cmds.StageOrUnstage)
 
-    def _open_using_default_app(self):
-        cmds.do(cmds.OpenDefaultApp, self.selected_group())
-
-    def _open_parent_dir(self):
-        cmds.do(cmds.OpenParentDir, self.selected_group())
-
     def show_selection(self):
         """Show the selected item."""
         # Sync the selection model
