@@ -1,11 +1,15 @@
 """build_pot command for setup.py"""
 
+from __future__  import absolute_import, division, unicode_literals
+
 import os
 import glob
 from distutils import log
 from distutils.core import Command
 from distutils.errors import DistutilsOptionError
 
+from . import build_util
+
 
 class build_pot(Command):
     """Distutils command build_pot"""
@@ -18,12 +22,11 @@ class build_pot(Command):
     #   - help string.
     user_options = [('build-dir=', 'd', 'Directory to put POT file'),
                     ('output=', 'o', 'POT filename'),
-                    ('lang=', None, 'Comma-separated list of languages '
-                                    'to update po-files'),
+                    ('lang=', None, 'Comma-separated list of languages to update po-files'),
                     ('no-lang', 'N', "Don't update po-files"),
-                    ('english', 'E', 'Regenerate English PO file'),
-                   ]
-    boolean_options = ['no-lang', 'english']
+                    ('english', 'E', 'Regenerate English PO file')]
+    user_options = build_util.stringify_options(user_options)
+    boolean_options = build_util.stringify_list(['no-lang', 'english'])
 
     def initialize_options(self):
         self.build_dir = None
@@ -53,7 +56,7 @@ class build_pot(Command):
             dst = src
         f = open(dst, 'wb')
         try:
-            f.write(content)
+            f.write(build_util.encode(content))
         finally:
             f.close()
 
