@@ -1,6 +1,7 @@
 #!/usr/bin/env python
-# -*- encoding: utf-8 -*-
-from __future__ import unicode_literals
+# encoding: utf-8
+
+from __future__ import absolute_import, division, unicode_literals
 import unittest
 
 from cola import core
